#include "EXTERN.h"
#include "perl.h"
#include "XSUB.h"

#include "ppport.h"

#include "ogdl.h"

SV *RvFromRv(SV *ptRv)
{
   return(newRV(SvRV(ptRv)));
}

SV *MakeHash(Graph ptGraph, SV *ptParent)
{
   static HV *ptGraphStash = NULL;
   HV *ptHash;
   SV *ptSubHash;
   AV *ptArray;
   SV *ptRef;
   int i;
   SV *ptResult;

   if (ptGraphStash == NULL)
   {
      ptGraphStash = gv_stashpv("OGDL::Graph", 0);
      if (!ptGraphStash)
         ptGraphStash = (HV *) -1;
   }

   ptHash = (HV *) newHV();

   if (!ptParent)
      /* Storing &PL_sv_undef does not work */
      hv_fetch(ptHash, "parent", 6, 1);
   else
      /* Store the reference to our parent, needs to be 
       * weak to prevent the circular reference killing 
       * the garbage collector */
      hv_store(ptHash, "parent", 6, sv_rvweaken(RvFromRv(ptParent)), 0);
   hv_store(ptHash, "name", 4, newSVpv(ptGraph->name, 0), 0);

   ptResult = newRV_noinc((SV *) ptHash);

   ptArray = newAV();
   av_extend(ptArray, ptGraph->size);
   for (i = 0; i < ptGraph->size; i++)
   {
      /* The return from MakeHash is already a 1 count 
       * reference to a 1 count hash, so we don't need 
       * to increment or otherwise change it's reference
       * count since we're its legitimate owner */
      ptSubHash = MakeHash(ptGraph->nodes[i], ptResult);
      av_push(ptArray, ptSubHash);
   }

   ptRef = newRV_noinc((SV *) ptArray);
   hv_store(ptHash, "list", 4, ptRef, 0);
   
   if (ptGraphStash && (ptGraphStash != (HV *) -1))
      sv_bless(ptResult, ptGraphStash);

   return(ptResult);
}

SV *GraphFile(char *sFile)
{
   SV *ptResult = NULL;
   HV *ptHash;
   FILE *ptFile;
   OgdlParser ptParser;

   ptFile = fopen(sFile, "r");
   if (!ptFile)
      return(NULL);
   
   ptParser = OgdlParser_new();
   if (!ptParser)
   {
      fclose(ptFile);
      return(NULL);
   }

   OgdlParser_parse(ptParser, ptFile);
   fclose(ptFile);

   if (ptParser->g && *ptParser->g)
   {
      ptResult = MakeHash(*ptParser->g, NULL);
   }

   if (!ptResult)
      ptResult = &PL_sv_undef;

   OgdlParser_free(ptParser);

   return(ptResult);
}

MODULE = OGDL::NativeParser		PACKAGE = OGDL::NativeParser		

SV * 
load(file)
      char *file
   PREINIT:
      Graph ptGraph = NULL;
   CODE:
      /* Create perl hashes to return */
      RETVAL = GraphFile(file);
   OUTPUT:
      RETVAL
      
      
      
      
      

