# Ogdl.pm
# author: R.Veen
# license: same as the rest of OGDL (zlib)
# see: www.ogdl.org
# date: 12 june 2003

package OGDL::Parser;

use strict;

our $VERSION = '0.01';

use OGDL::Graph;

sub LoadGraph{
    my ($name)=@_;
    if(-T "$name.gm"){
        return m4ToGraph("$name.gm");
    }
    if(-T "$name.g"){
        return fileToGraph("$name.g");
    }
    if(-T $name){
        return fileToGraph($name);
    }
    if(-d $name){
        return dirToGraph($name);
    }
    return undef;
}

sub dirToGraph{
    my ($dir)=@_;
    my $topdir=`pwd`;
    if(-d $dir){
        chdir $dir;
        my $g=OGDL::Graph->new($dir);
        my @dirlist=sort glob("*");
        foreach(@dirlist){
            if(-d $_){
        	my $subg=dirToGraph($_);
        	$g->addNode($subg);
            }
            else{
        	$g->addGraph($_);
            }
        }
#        print "chdir $cwd";
        chdir "..";
        return $g;
    }
    return undef;
}

sub fileToGraph
{
    open my $input, $_[0] or return undef;
    my $parser=OGDL::Parser->new($_[0]);
    $parser->read($input);
    my $g=$parser->parse;
    return $g;
}

sub m4ToGraph{
    open my $input, "m4 $_[0] |" or return undef;
    my $parser=OGDL::Parser->new($_[0]);
    $parser->read($input);
    my $g=$parser->parse;
    return $g;
}

# this routine accepts string
sub stringToGraph
{
    my $parser=OGDL::Parser->new("string");
    $parser->append($_[0]);
    my $g=$parser->parse;
    return $g;
}

# $ogdl->read(*FILEHANDL)
sub read{
    my $r_ogdl= $_[0];
    my $input =$_[1];
    while(<$input>) {
        _addline($r_ogdl, $_);
    }
}

sub _addline
{
    my ($p, $line) = @_;
    my $i;
    my $tabwidth = 8;

    $line =~ s/\r\n/\n/;           # Convert newlines
    $line =~ s/\s*$/\n/;	        # Remove any trailing spaces
    # Expand tabs, possibly intermixed
    # with spaces 
    while (($i = index($line, "\t", $i)) > -1)
    {
        my $n=$tabwidth-$i%$tabwidth;
        substr($line,$i,1)=' 'x$n;
        $i += $n;
    }
    push @{$p->{text}}, $line;
}

sub append{
    my $r_ogdl=$_[0];
    _addline($r_ogdl, $_ . "\n") for (split /\n/, $_[1])
}

sub new 
{
    my ($class,$rootname)=@_;
    my $tempg;
    $tempg=OGDL::Graph->new($rootname);
    my $p = {
      line => "",
      text => [],
      level => 0,
      indentation => [ () ],
      line_level => -1,
      groups => [ () ],
      ixgroup => 0,
#      allowgoback =>0, #temperary hack to decide whether comar will go back one indent level, not necessary with opposite precedence
      g => undef
   };
   $p->{g}[0]=$tempg;
   return bless $p, $class;
}

use constant PARSE_START => 0;

sub parse
{
    my $p = shift;
    my $line = 0;
    _nextifend($p);
    while (_line($p)) { }
    return $p->{g}[0];
}

sub _line
{
    my $p = shift;

# print "[" . substr ($p->{text},$p->{ix},20) . "]\n" ;      
    my $i = _space($p);
    if (_newline($p)) { return 1 };
    if (_eos($p)) { return 0 };
#print "line: lev(at entry) " . $p->{line_level} . "\n";
#print "line ind[]= ";
#$ind = $p->{indentation};
#for (@$ind) {
#print $_ . " ";
#}
#print ", i=$i, current ind=$p->{indentation}[$p->{line_level}];, line_level=$p->{line_level}\n";


    if ( $p->{line_level} == -1 ) { #start
        $p->{indentation}[0] = $i;
        $p->{line_level} = 0;
    }
    elsif ( $i > $p->{indentation}[$p->{line_level}] ) { #indentation increased
        $p->{line_level}++;
        $p->{indentation}[$p->{line_level}] = $i;
    }
    else {
        if ( $i < $p->{indentation}[$p->{line_level}] ) {#indentation decreased
            while ( $p->{line_level} > 0) { #find  the parent or brother line_level
                if ( $i >= $p->{indentation}[$p->{line_level}] ) {
                    last;
                }
                $p->{line_level}--;  
            }
        }
    }
    $p->{level} = $p->{line_level}; #current level
    $p->{groups}[$p->{ixgroup}]=$p->{level};
    $p->{allowgoback}=0;

# print "\ncurrent level is $p->{level}\n";    


# print "line: lev(after) " . $p->{line_level} . "\n";

    while ( _node($p) ) {
        _space($p); #Remove the trailing spaces
    }
    
    return _eos($p) ? 0:1;    
}

sub _node
{
#    print "get node\n";
    my $p = shift;
    if ( _eos($p) ) { return undef; }
    my $s=undef;
    my $c=_peek($p);
    if($c eq '('){
        _getc($p);
        $p->{ixgroup}++;
        $p->{groups}[$p->{ixgroup}] = $p->{level};
#        $p->{allowgoback}=0;
        return 1;
    }
    elsif($c eq ')'){
        _getc($p);
        $p->{level}=$p->{groups}[$p->{ixgroup}]+1;
#        $p->{allowgoback}=1;
        $p->{ixgroup}--;
        if ($p->{ixgroup} < 0) 
            { $p->_fatal("Unmatched ')'"); }
        return 1;
    }
    elsif($c eq ','){
        _getc($p);
        $p->{level} = $p->{groups}[$p->{ixgroup}];
#        if($p->{allowgoback}){
#            $p->{level}--; #exception: , at beginning
#            $p->{allowgoback}=0;
#        }
        return 1;
    }
    elsif($c eq "'" || $c eq '"'){
        $s=_quoted($p, '\'');
    }
    elsif($c eq '\\'){
        $s=_block($p);
    }
    elsif($c eq "\n"){
        $s=_newline($p);
        return 0;
    }
    elsif($c eq '#'){
        $s=_comment($p);
        return 0;
    }
    elsif($c eq 'w'){
        $s = _word($p);
    }
    else{
#        print "($c)" if defined $c;
        return undef;
    }

    if (defined $s) { # what if $s=""?
#        print $s;
        _addNode($p,$s);
        $p->{level}++;
#        $p->{allowgoback}=1;
        return 1;
    }
    
    #should never reach here.
    if(! _newline($p)){
        print "Strange? not newline, what else could it be?\n";
    }
    return 0;
}

sub _newline
{
    my $p = shift;
    if ($p->{line} =~ s/^\n//)
    {
        # Guaranteed to be a new line now 
        $p->{line} = shift @{$p->{text}};
        return(1);
    }
    return(0);
}

sub _eos
{
    my $p = shift;
    return 1 if (!(@{$p->{text}}) && ($p->{line} eq ""));
    return 0;
}

sub _space
{
    my $p = shift;
    my $n;
    if ($p->{line} =~ s/^([ \t]+)//)
    {
        $n = $1;
        _nextifend($p);
        return(length($n));
    }
    return(0);
}


sub _comment{#it will take out the new line as well
#   print "trying comment\n";
    my $p=$_[0];
    if ($p->{line} =~ s/^([^\n]*($|\n))//)
    {
        # Guaranteed to be a new line now 
        $p->{line} = shift @{$p->{text}};
        return($1);
    }
    return;
}

sub _quoted
{
    my $p = $_[0];
    my $s;

    if ( _eos($p) ) { return undef; }

    # We already know we're at a quote character 
    my $end = _getc($p);
    my $iteration = 0;
    my $indent;
    my ($part, $lastchar);
    
    while (1)
    {
        $iteration++;
        # Take any characters up to the end of the line or an end quote
        if ($p->{line} !~ s/^([^$end]*.?)//)
        {
            Carp::croak("{OGDL::Parser::_quoted} unfinished\n") if ($part eq "");
        }
        $part = $1;
        $lastchar = substr($part, -1, 1);
        _nextifend($p);

        # Append it to our out buffer 
        $s .= $part;

        # Keep looking if the last char (which must be either a 
        # newline of an end quote) is escaped 
        if ((reverse $part) !~ /^[^\\]?(\\\\)*\\([^\\]|$)/)
        {
            # If the last character is a quote, we're done
            if ($end eq $lastchar)
            {
                chop $s;
                last;
            }
            if ($lastchar eq "\n")
            {
                # Embedded newline, trim the current 
                # indent off the next line 
                $p->{line} =~ s/^( {0,$indent})//;
                _nextifend($p);

                # Now work out how long the indent of the 
                # next line is (i.e what we trimmed) and 
                # if it reduces our current indent
                my $newindent = $1;
                $newindent = length($newindent);
                $indent = $newindent if (($newindent < $indent) || (!defined($indent)));
            }
        }
    }

    # Unescape escaped strings etc 
    $s =~ s/\\([\n\\"'])/$1/g;

    return $s;
}

sub _block
{
    my $p = $_[0];

    if ( _eos($p) ) { return undef; }

    return undef if (!_matchandconsume($p, qr/\\\r?\n/));
    
    # what is the last current line indentation
    my $s;
    my $is=-1;
    my $re;
    my $empty = qr/[ \t]*(\r?\n|$)/m;
    
    while (1) {
        if ($is == -1)
        {
            # Get block indentation 
            $p->{line} =~ /^([ \t]*)/;
            # if less indented and not empty, exit
            if (length($1) < $p->{indentation}[$p->{line_level}])
            {
                my $match = _matchandconsume($p, $empty);
                # TODO: Do we add an empty line to the output?
                last if (!defined($match));
                next;
            }
            $is = length($1);
            $re = qr/[ \t]{$is}(.*?(\r?\n|$))/m;
        }

        last if (_eos($p));
        
        # Find a line with expected indent or greater 
        my ($all, $part) = _matchandconsume($p, $re);

        # Otherwise we'll accept an entirely blank line
        ($all, $part) = _matchandconsume($p, $empty) if (!defined($all));

        # If either matched, take and continue
        if (defined($all))
        {
            $s .= $part;
            next;
        }

        last;
    }

    # Strip any newline on the end, it's a separator
    $s =~ s/\r?\n$//;
    _ungetc($p, "\n") if (!_eos($p));
    return $s;
}

sub _nextSpace{
    my $p=shift;
    my $next=substr($p->{"line"},0,1);
    if(defined $next && $next eq ' ' || $next eq '\t'){
        return 1;
    }
    else {
        return 0;
    }
}
sub _nextWord{
    my $p=shift;
    my $next=substr($p->{"line"},0,2);
    if(defined $next && $next!~/^([ \t\r\n\(\),]|# )/ ){
        return 1;
    }
    else {
        return 0;
    }
}
sub _peek{
    my $p=shift;
    my $next=substr($p->{"line"},0,2);
    if(defined $next){
        if($next=~/^(['"\(\),]|# |\\[\r\n])/ ){
            return substr($next,0,1);
        }
        if($next=~/^[ \t]/ ){
            return " ";
        }
        if($next=~/^[\r\n]/ ){
            return "\n";
        }
        else{
            return 'w';
        }
    }
    return undef;
}

sub _word
{
    my $p = shift;
    if ($p->{line} =~ s/^([^ \t\r\n\(\),]+)//)
    {
        _nextifend($p);
        return($1);
    } 
    return;
}

sub _isSpaceChar
{
    if(defined $_[0]){
        if ($_[0] =~ /[ \t]/) { return 1; }
    }
    return 0;
}

sub _isWordChar
{
    if(defined $_[0]){
        if ($_[0] =~ /[ \t\n\r\(\),]/) { return 0; }
    }
    else{
        return 0;
    }
    return 1;
}

sub _getc
{
    my $p = shift;

    my $c = substr($p->{line},0,1);
    $p->{line} = substr($p->{line},1) if (defined($c));
    _nextifend($p);
    return $c;
}

sub _ungetc
{
    my ($p,$c) = @_;
    die "Wrong" if !defined($c);
    if ($c eq "\n")
    {
        unshift @{$p->{text}}, $p->{line};
        $p->{line} = "";
    }
    $p->{line} = $c . $p->{line};
}

sub _fatal
{
    my $p=shift;
    print "Fatal condition around [" . substr($p->{line}, 0, 10) . "]: ";
    print $_[0],"\n";
    exit(1);
}

sub _matchandconsume
{
    my ($p, $re) = @_;

    if (!defined(wantarray))
    {
        # Don't want any information about what happens
        $p->{line} =~ s/$re//;
        _nextifend($p);
        return;
    }

    # Need to return information about the result 
    if ($p->{line} =~ s/^($re)//)
    {
        _nextifend($p);
        return($1) if (!wantarray);
        my @result;
        do
        {
            no strict 'refs';
            my $i;
            push @result, ${$i} while (defined(${++$i}));
        };
        return @result;
    }

    return undef;
}

sub _nextifend
{
    my $p = shift;
    $p->{line} = shift @{$p->{text}} if ($p->{line} eq "");
}

sub _addNode
{
    my $p = shift;
    my $s = shift;

    unless ( $p->{g} ) {
        $p->{g}[0] = OGDL::Graph->new("stream"); 
    }
    
    my $g2 = OGDL::Graph->new($s);
    $p->{g}[$p->{level}]->addNode($g2);
    $p->{g}[$p->{level}+1] = $g2;
}

1;
__END__
=head1 NAME

OGDL::Parser - OGDL parser class

=head1 SYNOPSIS

  use OGDL::Parser;
  #Two easy interfaces
  $g=OGDL::Parser::fileToGraph($filename);
  #or parsing a string
  $g=OGDL::Parser::stringToGraph($string);

  #Another way to use the parser 
  #This method allows passing of any file handle to the parser
  $parser=OGDL::Parser->new;
  $parser->read(*STDIN);
  # $g is an OGDL::Graph object
  $g=$parser->parse;
  # print the whole graph.
  $g->print;

=head1 DESCRIPTION

OGDL possibly is a human editable alternative to  XML. For details
please see http://ogdl.sourceforge.net/. This class will parse an
ogdl conformed file or string into an OGDL::Graph object.

=head1 METHODS

$g=OGDL::Parser::fileToGraph($filename)
    This is a simple way of parsing an ogdl file. The method accepts 
    filename and returns an OGDL::Graph object parsed from the file.
    It returns undef if the parsing failed for any reason.

$g=OGDL::Parser::stringToGraph($string)
    This provides an easy way to parse an ogdl content held in memory.
    It accepts a string and returnes an OGDL::Graph object parsed from
    the string. It returns undef if the parsing failed.

$p=OGDL::Parser->new($rootname)
    This method constructs an OGDL::Parser object. If $rootname is 
    provided, it uses $rootname as the name of the rootnode for the 
    graph to parsed.

$p->read(*FILEHANDLE)
    This method reads in text into its internal buffer and make it 
    ready for parsing. The method can be used several times to read 
    in contents from different sources. The resulting ogdl graph will 
    be a plain combination of all the contents.

$p->append($str)
    This method adds $str to its internal buffer and make it ready for
    parsing.

$g=$p->parse
    This method parses its internal buffer and returns the OGDL::Graph
    object from parsing. It returns undef if parsing fails.

=head1 SEE ALSO
  OGDL::Graph, http://ogdl.sourceforge.net/.
=cut
